﻿package com {

	import flash.display.MovieClip;
	import flash.events.*;
	
	import com.greensock.*;
	import com.greensock.easing.*;
	
	/**
	* @author Michael Zalla
	*
	* The AlbumMenu class handles the animation and event triggering caused by the user's
	* interactions with the album's (visual) menu. The album menu consists of several text
	* buttons (e.g. – 'Home','Tracks',etc). The AlbumMenu class updates the visual appearance of
	* these buttons based on mouse events, and also handles Mouse.CLICK events for menu buttons
	*/
	public class AlbumMenu {

		public static var MENU_BUTTON_CLICKED:String = "MENU_BUTTON_CLICKED";

		private var _dispatcher:EventDispatcher;

		public var acceptingInput:Boolean = true;

		/* Menu buttons */
		private var _buttons:Array;
		public var currentButton:MovieClip;
		
		private var _home:MovieClip;
		private var _tracks:MovieClip;
		private var _lyrics:MovieClip;
		private var _artist:MovieClip;
		
		/* Baseline indicator */
		private var _indicator:MovieClip;
		// 										 state: [x, scaleX]
		private var _indicatorTweens:Object = {   home: [12,  0.95],
											    tracks: [80,  1.05],
											    //lyrics: [118, 0.95],
											    artist: [150, 0.95] };
		/**
		* Constructor
		* Main passes references to on-stage MovieClips when creating the StateMenu 
		* While StateMenu is responsible for menu behavior and appearance, Main is
		* responsible for which MovieClips become which menu buttons
		*/
		public function AlbumMenu(dispatcher_:EventDispatcher,home_,tracks_,lyrics_,artist_,indicator_) {
			
			_dispatcher = dispatcher_;
		
			_home = home_ as MovieClip;
			_tracks = tracks_ as MovieClip;
			_lyrics = lyrics_ as MovieClip;
			_artist = artist_ as MovieClip;
			
			/* Set up our handles (these are used by the click handler, and are also called when tweening the indicator) */
			_home.handle = "home";
			_tracks.handle = "tracks";
			_lyrics.handle = "lyrics";
			_artist.handle = "artist";
			
			//Now that each button is referencing a MovieClip on the stage, we can populate 'buttons'
			_buttons = [_home,_tracks,_lyrics,_artist];
			
			/* Attach mouseover behavior to menu items */
			for each(var button in _buttons) {
				button.buttonMode = true;
				button.addEventListener(MouseEvent.MOUSE_OVER,menuItemMouseover);
				button.addEventListener(MouseEvent.MOUSE_OUT,menuItemMouseover);
				button.addEventListener(MouseEvent.CLICK,handleMenuItemClick);
			}
			
			_indicator = indicator_ as MovieClip;
			
			/* Initialize currentButton to 'tracks', and call an initial 'update' */
			currentButton = _home;
			update();

		}

		/**
		* Handles mouseover behaviors for each of the menu's buttons
		* On mouseover, the item will tween to 1.00 alpha. On mouseout, the item
		* will tween to 0.45 alpha. The current menu item is ignored (no animation)
		*/
		private function menuItemMouseover(e:MouseEvent){
			var toggleAlpha:Object = { mouseOver: 1.00, mouseOut: 0.45 };
			var button:MovieClip = e.currentTarget as MovieClip;
			if(button != currentButton) {
				TweenMax.to(button,0.5,{ alpha:toggleAlpha[e.type],ease:Quint.easeOut });
			}
		}

		/**
		* Called by handleMenuItemClick after user clicks a menu button.
		* This function visually updates the menu by highlighting the newly current button,
		* and move-scales the baseline indicator to that button
		*/
		public function update():void {
			
			//Highlight the (new) current menu button
			for each(var button:MovieClip in _buttons)
				TweenMax.to(button,0.5,{ alpha: (button == currentButton) ? 1.00 : 0.45, ease:Quint.easeOut });

			if(currentButton.name != "m_item_Lyrics1") {
				//Move and scale the indicator to the (new) current menu button
				TweenMax.to(_indicator,1.0,{      x: _indicatorTweens[currentButton.handle][0],
											 scaleX: _indicatorTweens[currentButton.handle][1],
											   ease: Quint.easeOut });
			}
			
		}

		/**
		 * This function fires when the user clicks on a menu button. If the targeted button isn't the
		 * current button, then currentButton is updated, the menu is visually updated via update(),
		 * and a MENU_BUTTON_CLICKED event is dispatched. The Main class will listen for these events
		 * as cues for changing album states
		 */
		public function handleMenuItemClick(e):void {
			/* Only take input if acceptingInput is True */
			if(acceptingInput) {
				var targetButton:MovieClip = e.currentTarget as MovieClip;
				if(targetButton != currentButton) {
					currentButton = targetButton;
					update();
					//Lock additional input until TRANSITION_COMPLETE is recieved
					acceptingInput = false;
					_dispatcher.dispatchEvent(new Event(AlbumMenu.MENU_BUTTON_CLICKED));
				}
			}
		}

	}
	
}
