﻿package com  {

	import flash.display.MovieClip;
	import flash.events.*;   

	import com.greensock.*;
	import com.greensock.easing.*;
	
	/**
	* @author Michael Zalla
	* 
	* Objects based on this class act as handlers for groups of MovieClips that,
	* together, represent 'states' that the user navigates between. Three visual components
	* make up such as state: content, context and background. This class hold references to
	* each MovieClip, and supports methods for toggling each component on and off
	*/
	public class AlbumState {

		public static var TRANSITION_COMPLETE:String = "TRANSITION_COMPLETE";
		public static var CONTENT_COMPLETE:String = "CONTENT_COMPLETE";
		public static var CONTEXT_COMPLETE:String = "CONTEXT_COMPLETE";

		private var _dispatcher:EventDispatcher;

		public var handle:String;
		public var isCurrent:Boolean = false;
		public var contextMC:MovieClip;
		public var contentMC:MovieClip;
		public var backgroundMC:MovieClip;

		/**
		* Constructor
		* @param dispatcher_ A reference to an EventDispatcher inside of Main, used for cross-class Event listeners
		* @param handle_ A name (String) by which we can both refer to and compare the state(s)
		* @param contentMC_ A reference to the state's corresponding content (as a single MovieClip)
		* @param contextMC_ A reference to the state's corresponding context MovieClip
		* @param backgroundMC_ A reference to the state's corresponding background MovieClip
		*/
		public function AlbumState(dispatcher_:EventDispatcher,handle_:String,contentMC_,contextMC_,backgroundMC_):void {
			
			_dispatcher = dispatcher_;
			
			handle = handle_;		
			
			contentMC = contentMC_;
			contentMC.alpha = 0;			
			
			contextMC = contextMC_;
			contextMC.x = -108;

			backgroundMC = backgroundMC_;
			//Don't move the 'home' background, that's where the app will start!
			if(handle != 'home') backgroundMC.x = 501;
			
			trace("State '" + handle + "' created!");
			
		}

		/**
		* A. When called on the current AlbumState, that state's content is faded in (alpha = 1)
		*    – Dispatch an event that tells Main know that the state transition is complete
		* B. When called on a non-current AlbumState, that state's content is faded out (alpha = 0)
		*    – Dispatch an event to cause Main to show the new content
		*/
		public function toggleContent() {
			// A ...
			if(isCurrent) TweenMax.to(contentMC,2.0,{ alpha: 1, delay: 0.9, ease:Quint.easeOut, onComplete:onContentShowComplete });
			// B ...
			else TweenMax.to(contentMC,0.5,{ alpha: 0, ease:Quint.easeOut, onComplete:onContentHideComplete });
		}
		private function onContentHideComplete() {
			_dispatcher.dispatchEvent(new Event(AlbumState.CONTENT_COMPLETE));
		}
		private function onContentShowComplete() {
			_dispatcher.dispatchEvent(new Event(AlbumState.TRANSITION_COMPLETE));
		}
		
		/**
		* A. When called on the current AlbumState, that state's context is moved on-stage from the left
		* B. When called on a non-current AlbumState, that state's context is moved off-stage to the right (and reset)
		*    – Dispatch an event to cause Main to show the new context
		*/
		public function toggleContext() {
			// A ...
			if(isCurrent) TweenMax.to(contextMC,1.0,{ x:40, ease:Quint.easeOut });
			// B ...
			else TweenMax.to(contextMC,1.0,{ x:501, ease:Quint.easeOut, onComplete:onContextHideComplete });			
		}
		private function onContextHideComplete() {
			contextMC.x = -108; //Reset position to left-of-stage
			_dispatcher.dispatchEvent(new Event(AlbumState.CONTEXT_COMPLETE));
		}

		/**
		* A. When called on the current AlbumState, that state's background is moved on-stage from the left
		* B. When called on a non-current AlbumState, that state's background is moved off-stage to the right
		*/
		public function toggleBackground() {
			var bg:MovieClip = backgroundMC;
			// A ...
			if(isCurrent) TweenMax.to(bg,2.0,{ x: 0, ease:Quint.easeOut });
			// B ...
			else TweenMax.to(bg,2.0,{ x:-499, delay: 0.1, ease:Quint.easeOut, onComplete: function(){ bg.x = 501; } });

		}

	}
	
}