﻿package com {
	import flash.display.MovieClip;
	import flash.events.*;
	import flash.utils.*;
	
	import com.SongAlbum;
	import com.Scrollbar;
	
	import com.greensock.TweenLite;
	import com.greensock.easing.*;
	
	/**
	* @author Michael Zalla
	* 
	* This class defines an interface through which components (such as the playback buttons) can
	* control the playback of the SongAlbum. The methods inside PlaybackInterface can also be called
	* by classes like Main, for special cases (e.g. – the user clicking a track in the track list)
	*
	* The PlaybackInterface actually holds references to the playback button set, and handles their
	* behaviors as well
	*/
	public class PlaybackInterface {

		private var _album:SongAlbum;
		private var _buttons:Array;
		
		private var _controls:MovieClip;
		public var togglePlay:MovieClip;
		private var _previous:MovieClip;
		private var _next:MovieClip;

		private var _isPlaying:Boolean = false;

		private var _scrubber:MovieClip;
		private var _scrubberHandle:MovieClip

		/* Class constructor */
		public function PlaybackInterface(album_,scrubber_,controls_,hitbox_) {
			
			_album = album_;
			
			_controls = controls_ as MovieClip;
			togglePlay = _controls.playbackTogglePlay1 as MovieClip;
			_previous = _controls.playbackPrevious1 as MovieClip;
			_next = _controls.playbackNext1 as MovieClip;
			
			for each(var btn in [togglePlay,_previous,_next]) {
				btn.buttonMode = true;
				//Attach mouseover behaviors to next/previous buttons
				btn.addEventListener(MouseEvent.MOUSE_OVER, playbackButtonMouseover);
				btn.addEventListener(MouseEvent.MOUSE_OUT, playbackButtonMouseover);
				//All button events are routed to a single click handler
				btn.addEventListener(MouseEvent.CLICK,handlePlaybackControlClick);
			}
			
			_scrubber = scrubber_ as Scrollbar;
			/* Give the scrubber an onRelease function (updateSongPosition) */
			_scrubber.addOnRelease(updateSongPosition);
			/* Update the handle's position on the scrubber 5 times a second */
			var scrubberUpdate:uint = setInterval(updateHandlePosition, 200);

			/* Move the controls down below the Stage */
			//_controls.y = 500;

			/* Attach event listeners for showing/hiding the controls */
			//hitbox_.addEventListener(MouseEvent.MOUSE_OVER, toggleControls);
			//hitbox_.addEventListener(MouseEvent.MOUSE_OUT, toggleControls);
			
		}
		
		public function setIsPlaying(bool:Boolean):void {
			_isPlaying = bool;
		}
		
		/*
		private function toggleControls(e:MouseEvent = null) {
			if(e.type == "mouseOver") {
				TweenLite.to(_controls, 0.7, { y: 465, ease:Quint.easeOut });
			}
			else {
				TweenLite.to(_controls, 0.7, { y: 500, ease:Quint.easeOut });
			}
		}
		*/
		
		/**
		* Allows the PlaybackInterface to change the position of the SoundChannel
		* This function is then passed as the onRelease function for the scrubber's Scrollbar
		* Allows the user to manually move back and forth in the current song
		*/
		private function updateSongPosition(percentage):void {
			var duration = _album.getCurrentSongData().duration;
			var newPosition = int(duration * percentage); 
			_album.setPlayhead(newPosition);
		}
		
		/**
		* Get fired by scrubberUpdate (a setInterval variable) every 20 milliseconds
		* Scoots the scrubber handle across the track as the song progresses
		*/
		private function updateHandlePosition():void {
			if(!_scrubber.isScrolling) {
				var track:MovieClip = _scrubber.track as MovieClip;
				var handle:MovieClip = _scrubber.handle as MovieClip;
				//This line seems to break the appearance of manual scrolling (in the Scrollbar class)
				handle.x = _album.getCurrentSongPercentComplete() * (track.width-handle.width);
			}
		}
		
		/**
		* Controls the behavior of all playback buttons on mouseOver and mouseOut
		*/
		private function playbackButtonMouseover(e:MouseEvent) {
			var toggleAlpha:Object = { mouseOver: 1.00, mouseOut: 0.45 };
			var button:MovieClip = e.currentTarget as MovieClip;
			TweenLite.to(button,0.5,{ alpha:toggleAlpha[e.type],ease:Quint.easeOut });
		}
		
		/**
		* Fires when the user clicks a playback button on the stage
		* Controls the playback of the album. Also updates the appearance of the _playToggle button
		*/
		private function handlePlaybackControlClick(e):void {
			//Determine the targeted action, based on e.target
			var target:MovieClip = e.target as MovieClip;
			//If the user clicked on 'play'/'pause'
			if(target == togglePlay) {
				togglePlay.gotoAndPlay( (!_isPlaying) ? "play" : "pause" );
				_album[ (!_isPlaying) ? "play" : "pause" ]();
				_isPlaying = !_isPlaying;
			}
			//If the user clicked on 'previous'
			if(target == _previous) {
				_isPlaying = true;
				togglePlay.gotoAndStop("pause");
				_album.previous();
			}
			//If the user clicked on 'next'
			if(target == _next) {
				_isPlaying = true;
				togglePlay.gotoAndStop("pause");
				_album.next();
			}
		}

	}
	
}
