﻿package com  {
	
	import flash.display.MovieClip;
	import flash.events.*;
	
	/**
	* @author Michael Zalla
	* 
	* This class defines a Scrollbar object that is composed of one track MovieClip and one handle
	* MovieClip. The Scrollbar will maintain a scoll percentage value (between 0.0 and 1.0) that will
	* be used to update properties of external objects
	*/	
	public class Scrollbar extends MovieClip {

		protected var _track:MovieClip;
		protected var _handle:MovieClip;
		
		public var isScrolling:Boolean = false;
		protected var _scrollPercentage:Number = 0.0;
		
		protected var _onScroll:Function = null;
		protected var _onRelease:Function = null;
		
		/**
		* Class constructor 
		*/
		public function Scrollbar() {
			_track = this.getChildAt(0) as MovieClip;
			_handle = this.getChildAt(1) as MovieClip;
			this.addEventListener(Event.ADDED_TO_STAGE, addedToStageHandler);
		}

		/* Listeners have to be attached after the Scrollbar has been added to the stage */
		protected function addedToStageHandler(e:Event = null):void {
			
			//Mouse is down. Start listening for any scrolling!
			_handle.addEventListener(MouseEvent.MOUSE_DOWN, function(e){
				isScrolling = true;
				stage.addEventListener(MouseEvent.MOUSE_MOVE, scroll);
			});

			// Mouse has been released. No more scrolling!
			stage.addEventListener(MouseEvent.MOUSE_UP, function(e){
				if(isScrolling) {
					isScrolling = false;
					stage.removeEventListener(MouseEvent.MOUSE_MOVE, scroll);
					//Execute the mouseUpFunction, if it has been set
					if(_onRelease != null) _onRelease(_scrollPercentage);
				}
			});

		}
		
		/* Updates _scrollPercentage when the handle is moved, and also executes the _onScroll function (if set) */
		protected function scroll(e:MouseEvent):void {
			var mousePosition:Number = this.mouseX;
			_scrollPercentage = mousePosition/_track.width;
			//Preserve scroll boundaries
			if(_scrollPercentage < 0) _scrollPercentage = 0;
			if(_scrollPercentage > 1) _scrollPercentage = 1;
			//Execute _onScroll
			if(_onScroll != null) _onScroll(_scrollPercentage); //Keep in mind this is a function call!
			//Move the handle along with the mouse
			_handle.x = _scrollPercentage * (_track.width - _handle.width);
		}

		/******** PUBLIC API ********/
		
		/* Public getter for _scrollPercentage */
		public function getScrollPercentage():Number { return _scrollPercentage; }
		
		/* Allows you to specify a new position for the scroll handle without a mouse event */
		public function setScrollPercentage(percentage:Number):void {
			if(percentage < 0) percentage = 0;
			if(percentage > 1) percentage = 1;
			_scrollPercentage = percentage
			//Update the handle's position
			_handle.x = _scrollPercentage * (_track.width - _handle.width);
		}
		
		/* Allows you to pass an external function to Scrollbar that will fire when the handle moves */
		public function addOnScroll(fn:Function):void { _onScroll = fn; }
		
		/* Allows you to pass an external function to Scrollbar that will fire when the handle is released */
		public function addOnRelease(fn:Function):void { _onRelease = fn; }

	}
	
}
